'use strict';Object.defineProperty(exports,'__esModule',{value:true});var chessground=require('chessground'),chessops=require('@publishvue/chessops');function simpleFetch(url, params, callback) {
    params.headers = params.headers || {};
    if (params.asForm)
      { params.headers["Content-Type"] = "application/x-www-form-urlencoded"; }
    if (params.asJson) { params.headers.Accept = "application/json"; }
    if (params.asVndLichessV3Json) {
      params.headers.Accept = "application/vnd.lichess.v3+json";
      params.asJson = true;
    }
    if (params.asNdjson) { params.headers.Accept = "application/x-ndjson"; }
    if (params.accessToken)
      { params.headers.Authorization = "Bearer " + params.accessToken; }
    if (params.server)
      { api(
        "request:fetch",
        {
          url: url,
          params: params
        },
        function (result) { return callback(result); }
      ); }
    else
      { fetch(url, params).then(
        function (response) { return response.text().then(
            function (text) {
              if (params.asJson || params.asNdjson) {
                try {
                  var obj;
                  if (params.asNdjson) {
                    obj = text
                      .split("\n")
                      .filter(function (line) { return line.length; })
                      .map(function (line) { return JSON.parse(line); });
                  } else {
                    obj = JSON.parse(text);
                  }
                  try {
                    callback({ ok: true, content: obj });
                  } catch (err) {
                    console.log(err, obj);
                  }
                } catch (err) {
                  console.log("fetch parse json error", err);
                  callback({ ok: false, status: "Error: Could not parse json." });
                }
              } else {
                callback({ ok: true, content: text });
              }
            },
            function (err) {
              console.log("fetch get response text error", err);
              callback({
                ok: false,
                status: "Error: Failed to get response text."
              });
            }
          ); },
        function (err) {
          console.log("fetch error", err);
          callback({ ok: false, status: "Error: Failed to fetch." });
        }
      ); }
  }var LICHESS_BOOK_URL = "https://explorer.lichess.ovh/lichess";

var LICHESS_BOOK_MAX_MOVES = 12;
var LICHESS_BOOK_AVG_RATINGS = [1600, 1800, 2000, 2200, 2500];
var LICHESS_BOOK_TIME_CONTROLS = ["bullet", "blitz", "rapid", "classical"];

var STANDARD_START_FEN =
  "rnbqkbnr/pppppppp/8/8/8/8/PPPPPPPP/RNBQKBNR w KQkq - 0 1";

var DEFAULT_VARIANT = "standard";

var bookCache = {};

function requestLichessBook(
  fenOpt,
  variantOpt,
  maxMovesOpt,
  ratingListOpt,
  speedListOpt
) {
  var fen = fenOpt || STANDARD_START_FEN;
  var variant = variantOpt || DEFAULT_VARIANT;
  var maxMoves = maxMovesOpt || LICHESS_BOOK_MAX_MOVES;
  var ratingList = ratingListOpt || LICHESS_BOOK_AVG_RATINGS;
  var speedList = speedListOpt || LICHESS_BOOK_TIME_CONTROLS;

  var key = fen + "|" + variant + "|" + maxMoves + "|" + ratingList + "|" + speedList;

  if (bookCache[key]) {
    //console.log("book position found in cache for key", key);
    return Promise.resolve(bookCache[key]);
  }

  var ratings = ratingList.map(function (opt) { return ("ratings%5B%5D=" + opt); }).join("&");

  var speeds = speedList.map(function (opt) { return ("speeds%5B%5D=" + opt); }).join("&");

  var url =
    LICHESS_BOOK_URL + "?fen=" + fen + "&moves=" + maxMoves + "&variant=" + variant;

  if (ratings) { url += "&" + ratings; }

  if (speeds) { url += "&" + speeds; }

  return new Promise(function (resolve) {
    simpleFetch(
      url,
      {
        asJson: true
      },
      function (result) {
        if (result.ok) {
          result.content.fen = fen;
          bookCache[key] = result.content;
          resolve(result.content);
        }
      }
    );
  });
}

function weightedRandom(book) {
  if (!book) { return null; }

  if (!book.moves.length) { return null; }

  var sum = 0;

  for (var i$1 = 0, list = book.moves; i$1 < list.length; i$1 += 1) {
    var move = list[i$1];

    move.total = move.white + move.draws + move.black;
    sum += move.total;
  }

  var rnd = Math.floor(Math.random() * sum);

  var ctotal = book.moves[0].total;

  var i = 0;

  while (ctotal < rnd) {
    ctotal += book.moves[i++].total;
  }

  var selectedMove = book.moves[i];

  return selectedMove;
}

function makeVariant(display, chessops, explorer) {
  return {
    display: display,
    chessops: chessops,
    explorer: explorer
  };
}

var LICHESS_VARIANTS = [
  makeVariant("Standard", "chess", "standard"),
  makeVariant("Antichess", "antichess", "antichess"),
  makeVariant("Atomic", "atomic", "atomic"),
  makeVariant("Crazyhouse", "crazyhouse", "crazyhouse"),
  makeVariant("Horde", "horde", "horde"),
  makeVariant("King of the Hill", "kingofthehill", "kingOfTheHill"),
  makeVariant("Racing Kings", "racingkings", "racingKings")
];

function chessopsToExplorer(variant) {
  var search = LICHESS_VARIANTS.find(function (test) { return test.chessops === variant; });
  return search ? search.explorer : "standard";
}//

var script = {
  name: "Vuechessground",
  data: function data(){
    var pos = chessops.Pos();
    return {      
      pos: pos,
      board: null,
      legalsans: [],
      history: [],      
      variants: [
        {display: "Standard", key: "standard"},
        {display: "Atomic", key: "atomic"},
        {display: "Antichess", key: "antichess"},
        {display: "Racing Kings", key: "racingkings"},
        {display: "Horde", key: "horde"}
      ],
      promuci: null,
      showpromotion: false,
      prompieces: [],
      prompiecesext: [],
      currentbook: null,
      bookmoves: [],
    }
  },
  methods:{
    getBook: function getBook(){
      var this$1$1 = this;
       
      this.currentbook = null;
      this.bookmoves = [];
      requestLichessBook(
        this.pos.reportFen(),
        chessopsToExplorer(this.pos.pos.rules)
      ).then(function (result) {
        //console.log("book", result);
        this$1$1.currentbook = result;
        this$1$1.bookmoves = result.moves;        
      });
    },
    makeWeightedrandom: function makeWeightedrandom() {
      if (this.currentbook) {
        if (this.currentbook.fen === this.pos.reportFen()) {
          var selectedMove = weightedRandom(this.currentbook);

          if (!selectedMove) {
            //console.log("no book move for the position");

            return;
          }

          var san = selectedMove.san;

          this.makesanmove(null, san);
        }
      }
    },
    makemovecancel: function makemovecancel(){
      this.prompieces = [];
      this.prompiecesext = [];
      this.$refs.movetextinput.value = "";
      this.setUp();
    },
    pieceToName: function pieceToName(p){
      return {
        q: "queen",
        r: "rook",
        b: "bishop",
        n: "knight",
        k: "king",
        p: "pawn"
      }[p]
    },
    pieceToStyle: function pieceToStyle(p, fen){
      var parts = fen.split(" ");
      var turn = parts[1] == "w" ? "white" : "black";
      var name = this.pieceToName(p);
      var style = "piece " + name + " " + turn;      
      return style
    },
    step: function step(dir){
      var fen = this.pos.reportFen();
      //console.log("step", dir, fen)
      var index = this.history.findIndex(function (item) { return item.fen === fen; });
      if(index < 0){
        //console.error("fen not found", fen)
        return
      }
      index += dir;
      if(index < 0) { index = 0; }
      if(index >= this.history.length) { index = this.history.length - 1; }
      this.pos.setFen(this.history[index].fen);
      this.getSelectedHistory();
      this.setUp();
      this.prompieces = [];
      this.prompiecesext = [];
    },
    tobegin: function tobegin(){
      this.step(-100);
    },
    back: function back(){
      this.step(-1);
    },
    forward: function forward(){
      this.step(1);
    },
    toend: function toend(){
      this.step(100);
    },
    del: function del(){
      this.back();
      var fen = this.pos.reportFen();
      //console.log("step", dir, fen)
      var index = this.history.findIndex(function (item) { return item.fen === fen; });
      if(index >= 0){
        this.history = this.history.slice(0, index + 1);
      }
    },
    clickhistory: function clickhistory(ev) {
      var fen = ev.target.getAttribute("fen");      
      this.pos.setFen(fen);
      this.getSelectedHistory();            
      var itemIndex = this.history.findIndex(function (item) { return item.selected == "histactive"; });
      //console.log(itemIndex, this.history[itemIndex])
      this.recordMovePlayed(this.history[itemIndex].genSan, itemIndex > 0 ? this.history[itemIndex - 1].fen : null, true);
    },
    getSelectedHistory: function getSelectedHistory(){
      var fen = this.pos.reportFen();
      for(var i = 0, list = this.history; i < list.length; i += 1) {
        var item = list[i];

        item.selected = item.fen == fen ? "histactive" : "histpassive";
      }
    },
    recordMovePlayed: function recordMovePlayed(san, prevFen, dontAdd){      
      var fen = this.pos.reportFen();            

      if(!dontAdd){
        var index = this.history.findIndex(function (item) { return item.fen === prevFen; });
        
        if(index < 0){        
          //console.log("previous fen not in history")
          this.history.push({ genSan: san, fen: fen });        
        }else {
          //console.log("previous in history, trimming")
          this.history = this.history.slice(0, index + 1);
          this.history.push({ genSan: san, fen: fen });
        }
      }

      this.getSelectedHistory();      
      this.prompieces = [];
      this.prompiecesext = [];
      this.setUp();

      var event = {
        kind: "moveplayed",
        prevFen: prevFen,
        fen: fen,
        genSan: san,
        variant: this.pos.pos.rules,
        history: this.history,
      };

      this.$emit("moveplayed", event);
    },
    prompiececlicked: function prompiececlicked(ev){
      var piece = ev.target.getAttribute("piece");
      var uci = this.promuci + piece;
      var san = this.pos.uciToSan(uci);
      this.$refs.movetextinput.value = san;
      this.showpromotion = false;
    },
    makesanmove: function makesanmove(ev, san){
      this.$refs.movetextinput.value = san;
      this.makemove();
    },
    makemove: function makemove(){
      var fen = this.pos.reportFen();
      var move = this.$refs.movetextinput.value;
      this.$refs.movetextinput.value = "";
      try {
        this.pos.playSan(move);
        this.recordMovePlayed(move, fen);
      }catch(err){
        try{
          var san = this.pos.uciToSan(move);
          this.pos.playSan(san);
          this.recordMovePlayed(san, fen);          
        }catch(err){
          window.alert(("Illegal move " + move + " !"));
          return
        }
      }
    },
    variantchanged: function variantchanged(){      
      var variant = this.$refs.variantselect.value;      
      this.pos.setVariant(variant);      
      this.history = [];      
      this.recordMovePlayed("*", null);        
    },
    reset: function reset(){
      this.variantchanged();
    },
    clicksan: function clicksan(ev){
      var fen = this.pos.reportFen();
      var san = ev.target.getAttribute("san");      
      this.pos.playSan(san);
      this.recordMovePlayed(san, fen);
    },
    setUp: function setUp(){
      var fen = this.pos.reportFen();
      this.$refs.fentextinput.value = fen;
      var config = {
          fen: fen,        
          movable: { events: { after: this.movePlayed() } },
      };
      this.board = chessground.Chessground(this.$refs.board, config);      
      this.legalsans = this.pos.allLegalSans();      
      this.getBook();
    },
    movePlayed: function movePlayed(){
      var this$1$1 = this;

      return function (orig, dest, metadata) {
        var fen = this$1$1.pos.reportFen();
        var uci = "" + orig + dest;        
        var legals = this$1$1.pos.legalsForUci(uci);
        var num = legals.length;
        if(!num){
          window.alert(("Illegal move " + uci + " !"));
          this$1$1.$refs.movetextinput.value = "";
          this$1$1.showpromotion = false;
          this$1$1.setUp();
          return
        }
        if(num > 1){
          // handle promotion
          this$1$1.prompieces = legals.map(function (legal) { return legal.substring(4, 5); });
          this$1$1.prompiecesext = this$1$1.prompieces.map(function (p) { return ({
            p: p,
            klass: "prompiecediv " + this$1$1.pieceToStyle(p, this$1$1.pos.reportFen())              
          }); });
          this$1$1.showpromotion = true;          
          this$1$1.$refs.movetextinput.value = "Press piece !";
          this$1$1.promuci = uci;          
          return
        }
        this$1$1.$refs.movetextinput.value = "";
        this$1$1.showpromotion = false;
        var san = this$1$1.pos.uciToSan(uci);
        this$1$1.pos.playUci(uci);        
        this$1$1.recordMovePlayed(san, fen);        
      }
    }
  },
  mounted: function mounted(){    
    this.variantchanged();    
  },
  components:{

  }
};function normalizeComponent(template, style, script, scopeId, isFunctionalTemplate, moduleIdentifier /* server only */, shadowMode, createInjector, createInjectorSSR, createInjectorShadow) {
    if (typeof shadowMode !== 'boolean') {
        createInjectorSSR = createInjector;
        createInjector = shadowMode;
        shadowMode = false;
    }
    // Vue.extend constructor export interop.
    var options = typeof script === 'function' ? script.options : script;
    // render functions
    if (template && template.render) {
        options.render = template.render;
        options.staticRenderFns = template.staticRenderFns;
        options._compiled = true;
        // functional template
        if (isFunctionalTemplate) {
            options.functional = true;
        }
    }
    // scopedId
    if (scopeId) {
        options._scopeId = scopeId;
    }
    var hook;
    if (moduleIdentifier) {
        // server build
        hook = function (context) {
            // 2.3 injection
            context =
                context || // cached call
                    (this.$vnode && this.$vnode.ssrContext) || // stateful
                    (this.parent && this.parent.$vnode && this.parent.$vnode.ssrContext); // functional
            // 2.2 with runInNewContext: true
            if (!context && typeof __VUE_SSR_CONTEXT__ !== 'undefined') {
                context = __VUE_SSR_CONTEXT__;
            }
            // inject component styles
            if (style) {
                style.call(this, createInjectorSSR(context));
            }
            // register component module identifier for async chunk inference
            if (context && context._registeredComponents) {
                context._registeredComponents.add(moduleIdentifier);
            }
        };
        // used by ssr in case component is cached and beforeCreate
        // never gets called
        options._ssrRegister = hook;
    }
    else if (style) {
        hook = shadowMode
            ? function (context) {
                style.call(this, createInjectorShadow(context, this.$root.$options.shadowRoot));
            }
            : function (context) {
                style.call(this, createInjector(context));
            };
    }
    if (hook) {
        if (options.functional) {
            // register for functional component in vue file
            var originalRender = options.render;
            options.render = function renderWithStyleInjection(h, context) {
                hook.call(context);
                return originalRender(h, context);
            };
        }
        else {
            // inject component registration as beforeCreate hook
            var existing = options.beforeCreate;
            options.beforeCreate = existing ? [].concat(existing, hook) : [hook];
        }
    }
    return script;
}/* script */
var __vue_script__ = script;
/* template */
var __vue_render__ = function () {var _vm=this;var _h=_vm.$createElement;var _c=_vm._self._c||_h;return _c('div',{staticClass:"vuechessground"},[_vm._ssrNode("<div class=\"horizcont\"><div class=\"vertcont\"><div class=\"moveinputdiv\"><input type=\"text\" class=\"movetextinput\"> "+((_vm.showpromotion)?("<div class=\"prompiecebuttons\">"+(_vm._ssrList((_vm.prompiecesext),function(prompieceext){return ("<div"+(_vm._ssrAttr("piece",prompieceext.p))+(_vm._ssrClass(null,prompieceext.klass))+">"+_vm._ssrEscape("\n            "+_vm._s(prompieceext.p.toUpperCase())+"\n          ")+"</div>")}))+"</div>"):"<!---->")+" <button>Make Move</button> <button>Cancel</button> <button>Reset</button></div> <div class=\"blue merida\"><div class=\"cg-board-wrap\"></div></div> <div class=\"fendiv\"><input tytpe=\"text\" class=\"fentextinput\"></div> <div class=\"controls\"><button>|&lt;</button> <button>&lt;</button> <button>&gt;</button> <button>&gt;|</button> <button>X</button> <select>"+(_vm._ssrList((_vm.variants),function(variant){return ("<option"+(_vm._ssrAttr("value",variant.key))+">"+_vm._ssrEscape(_vm._s(variant.display))+"</option>")}))+"</select> <button>Rand</button></div></div> <div class=\"legalsans\">"+(_vm._ssrList((_vm.legalsans),function(item){return ("<div><div"+(_vm._ssrAttr("san",item))+" class=\"legalsan\">"+_vm._ssrEscape("\n          "+_vm._s(item)+"\n        ")+"</div></div>")}))+"</div> <div class=\"historysans\">"+(_vm._ssrList((_vm.history),function(item){return ("<div"+(_vm._ssrAttr("fen",item.fen))+(_vm._ssrClass("historysan",item.selected))+">"+_vm._ssrEscape("            \n        "+_vm._s(item.genSan)+"          \n      ")+"</div>")}))+"</div> <div class=\"bookdiv\"><table cellpadding=\"3\" cellspacing=\"3\"><thead><tr><td class=\"bookmovesan\">Move</td> <td class=\"white\">White Wins</td> <td class=\"draw\">Draw</td> <td class=\"black\">Black Wins</td></tr></thead> <tbody>"+(_vm._ssrList((_vm.bookmoves),function(item){return ("<tr><td class=\"bookmovesan\">"+_vm._ssrEscape("\n              "+_vm._s(item.san)+"\n            ")+"</td> <td class=\"white\">"+_vm._ssrEscape(_vm._s(item.white))+"</td> <td class=\"draw\">"+_vm._ssrEscape(_vm._s(item.draws))+"</td> <td class=\"black\">"+_vm._ssrEscape(_vm._s(item.black))+"</td></tr>")}))+"</tbody></table></div></div>")])};
var __vue_staticRenderFns__ = [];

  /* style */
  var __vue_inject_styles__ = undefined;
  /* scoped */
  var __vue_scope_id__ = undefined;
  /* module identifier */
  var __vue_module_identifier__ = "data-v-9af2ed60";
  /* functional template */
  var __vue_is_functional_template__ = false;
  /* style inject */
  
  /* style inject SSR */
  
  /* style inject shadow dom */
  

  
  var __vue_component__ = /*#__PURE__*/normalizeComponent(
    { render: __vue_render__, staticRenderFns: __vue_staticRenderFns__ },
    __vue_inject_styles__,
    __vue_script__,
    __vue_scope_id__,
    __vue_is_functional_template__,
    __vue_module_identifier__,
    false,
    undefined,
    undefined,
    undefined
  );// install function executed by Vue.use()
function install(Vue) {
  if (install.installed) { return; }
  install.installed = true;
  Vue.component('Vuechessground', __vue_component__);
}

// Create module definition for Vue.use()
var plugin = {
  install: install,
};

// To auto-install when vue is found
var GlobalVue = null;
if (typeof window !== 'undefined') {
  GlobalVue = window.Vue;
} else if (typeof global !== 'undefined') {
  GlobalVue = global.Vue;
}
if (GlobalVue) {
  GlobalVue.use(plugin);
}

// Inject install function into component - allows component
// to be registered via Vue.use() as well as Vue.component()
__vue_component__.install = install;

// It's possible to expose named exports when writing components that can
// also be used as directives, etc. - eg. import { RollupDemoDirective } from 'rollup-demo';
// export const RollupDemoDirective = component;
exports["default"]=__vue_component__;//# sourceMappingURL=vuechessground.ssr.js.map
